# -*- coding: utf-8 -*-
"""
Created on Fri Feb 18 11:05:22 2022

@author: pwiechen
"""
#%%
import numpy as np   
import pandas as pd            
import matplotlib.pyplot as plt 
import os
import sys
from datetime import datetime, timedelta
import netCDF4 as nc4

%matplotlib qt 

dir_path = os.path.dirname(os.path.realpath(__file__))
os.chdir(dir_path)

print(dir_path)
#%% Instrument coordinates

types_dict = {'description': str,'date':str, 'time_raw':str, 'x_val': float, 'y_val': float, 'z_val': float, '2d_error': float, '3d_error': float}
max_error = 0.05 

instrument_height_rbr = 0.215
instrument_height_jfe = 0.124

gps_20220105 = pd.read_csv(r'../../GPS/20220105/20220105_INSTRUMENTS.txt', usecols=[0,1,2,3,4,5,6,7], index_col=False,delim_whitespace=True, skipinitialspace=True, names=['description','date', 'time_raw', 'x_val', 'y_val', 'z_val', '2d_error', '3d_error'], dtype=types_dict, parse_dates={'time':['date', 'time_raw']})
gps_20220105 = gps_20220105.drop(gps_20220105[gps_20220105['3d_error'] > max_error].index)

gps_20220106 = pd.read_csv(r'../../GPS/20220106/20220106_INSTRUMENTS.txt', usecols=[0,1,2,3,4,5,6,7], index_col=False,delim_whitespace=True, skipinitialspace=True, names=['description','date', 'time_raw', 'x_val', 'y_val', 'z_val', '2d_error', '3d_error'], dtype=types_dict, parse_dates={'time':['date', 'time_raw']})
gps_20220106 = gps_20220106.drop(gps_20220106[gps_20220106['3d_error'] > max_error].index)

gps_20220107 = pd.read_csv(r'../../GPS/20220107/20220107_INSTRUMENTS.txt', usecols=[0,1,2,3,4,5,6,7], index_col=False,delim_whitespace=True, skipinitialspace=True, names=['description','date', 'time_raw', 'x_val', 'y_val', 'z_val', '2d_error', '3d_error'], dtype=types_dict, parse_dates={'time':['date', 'time_raw']})
gps_20220107 = gps_20220107.drop(gps_20220107[gps_20220107['3d_error'] > max_error].index)

gps_tot = pd.concat([gps_20220105,gps_20220106,gps_20220107])

S01_RBR_01_gps = np.mean(gps_tot[gps_tot.description.str.contains('S01_RBR_01_TOP')].values[:,2:], axis=0)
S01_RBR_01_gps[2] -= instrument_height_rbr

S02_RBR_02_gps = np.mean(gps_tot[gps_tot.description.str.contains('S02_RBR_02_TOP')].values[:,2:], axis=0)
S02_RBR_02_gps[2] -= instrument_height_rbr

S03_RBR_03_gps = np.mean(gps_tot[gps_tot.description.str.contains('S03_RBR_03_TOP')].values[:,2:], axis=0)
S03_RBR_03_gps[2] -= instrument_height_rbr

S04_RBR_04_gps = np.mean(gps_tot[gps_tot.description.str.contains('S04_RBR_04_TOP')].values[:,2:], axis=0)
S04_RBR_04_gps[2] -= instrument_height_rbr

S05_RBR_05_gps = np.mean(gps_tot[gps_tot.description.str.contains('S05_RBR_05_TOP')].values[:,2:], axis=0)
S05_RBR_05_gps[2] -= instrument_height_rbr

S06_RBR_06_gps = np.mean(gps_tot[gps_tot.description.str.contains('S06_RBR_06_TOP')].values[:,2:], axis=0)
S06_RBR_06_gps[2] -= instrument_height_rbr

S01_JFE_01_gps = np.mean(gps_tot[gps_tot.description.str.contains('S01_JFE_01_TOP')].values[:,2:], axis=0)
S01_JFE_01_gps[2] -= instrument_height_jfe

S02_JFE_02_gps = np.mean(gps_tot[gps_tot.description.str.contains('S02_JFE_02_TOP')].values[:,2:], axis=0)
S02_JFE_02_gps[2] -= instrument_height_jfe

S04_JFE_03_gps = np.mean(gps_tot[gps_tot.description.str.contains('S04_JFE_03_TOP')].values[:,2:], axis=0)
S04_JFE_03_gps[2] -= instrument_height_jfe

S05_JFE_04_gps = np.mean(gps_tot[gps_tot.description.str.contains('S05_JFE_04_TOP')].values[:,2:], axis=0)
S05_JFE_04_gps[2] -= instrument_height_jfe

#%%
print(S01_RBR_01_gps)
#%%

'''
JANUARY 5: 13:00
S01JFE1: 25
S01RBR1: 12
S02JFE2: 17
S02RBR2: 06
S03RBR3:-17.5
S04JFE3: 23
S04RBR4: 34
S05JFE4: 29
S05RBR5: 18
S06RBR6:-18

JANUARY 6: 10:00
S01JFE1: 20
S01RBR1:  5
S02JFE2: 10
S02RBR2: -5
S03RBR3:-20
S04JFE3: 28
S04RBR4: 14
S05JFE4: 35
S05RBR5: 25
S06RBR6:  1

JANUARY 7: 12:00
S01JFE1: 25 
S01RBR1: 9
S02JFE2: 7
S02RBR2: -7
S03RBR3: -21
S04JFE3: 38
S04RBR4: 27
S05JFE4: 38
S05RBR5: 30
S06RBR6: 0

'''
#%% Creation of Netcdf

time_surveys = np.array((datetime(2022,1,5,14,0,0).timestamp(),
                         datetime(2022,1,6,11,30,0).timestamp(),
                         datetime(2022,1,7,13,0,0).timestamp()))

print(pd.to_datetime(time_surveys, unit='s').tz_localize('GMT').tz_convert('CET'))

#%%
instrument_heights_jan_2022 = nc4.Dataset(r'20220105_20220107_instrument_heights.nc','w', format='NETCDF4')

instrument_heights_jan_2022.description = 'Instrument coordinates and heights for instruments deployed between 2022-01-05 00:00:00 – 2022-01-08 00:00:00, for the Realdune - Reflex experiments on the Sand Engine, the Netherlands'

today = datetime.today()
instrument_heights_jan_2022.history = "Created " + today.strftime("%y-%m-%d")

instrument_heights_jan_2022.createDimension('loc', 0)
instrument_heights_jan_2022.createDimension('time', len(time_surveys))

time = instrument_heights_jan_2022.createVariable('time', 'i8', 'time')
time[:] = time_surveys

###### S01_RBR_01 ######

S01_RBR_01 = instrument_heights_jan_2022.createGroup('S01_RBR_01')
S01_RBR_01.description = 'RDNAP coordinates and heights above the bed for S01_RBR_01'

time = S01_RBR_01.createVariable('time', 'i8', 'time')
x = S01_RBR_01.createVariable('x', 'g', 'loc')  
y = S01_RBR_01.createVariable('y', 'g', 'loc')
z = S01_RBR_01.createVariable('z', 'g', 'loc')
h_bed = S01_RBR_01.createVariable('h_bed', 'g', 'time')  

x[:] = '%.4f' % round(S01_RBR_01_gps[0],4)
y[:] = '%.4f' % round(S01_RBR_01_gps[1],4)
z[:] = '%.4f' % round(S01_RBR_01_gps[2],4)

time[:] = time_surveys
h_bed[:] = np.array([12, 5, 9])/100 

time.description = 'time in unix epoch time (seconds after 1970-01-01 00:00:00 UTC)'
x.description = 'x-coordinate RDNAP for S01_RBR_01 [m]'
y.description = 'y-coordinate RDNAP for S01_RBR_01 [m]'
z.description = 'z-coordinate RDNAP for S01_RBR_01 [m]'
h_bed.description = 'height above the bed of the sensor of S01_RBR_01 [m]'

time.units = 's'
x.units = 'm'
y.units = 'm'
z.units = 'm'
h_bed.units = 'm'

S02_RBR_02 = instrument_heights_jan_2022.createGroup('S02_RBR_02')
S02_RBR_02.description = 'RDNAP coordinates and heights above the bed for S02_RBR_02'

time = S02_RBR_02.createVariable('time', 'i8', 'time')
x = S02_RBR_02.createVariable('x', 'g', 'loc')  
y = S02_RBR_02.createVariable('y', 'g', 'loc')
z = S02_RBR_02.createVariable('z', 'g', 'loc')
h_bed = S02_RBR_02.createVariable('h_bed', 'g', 'time')  

x[:] = '%.4f' % round(S02_RBR_02_gps[0],4)
y[:] = '%.4f' % round(S02_RBR_02_gps[1],4)
z[:] = '%.4f' % round(S02_RBR_02_gps[2],4)

time[:] = time_surveys
h_bed[:] = np.array([6,-5,-7])/100 

time.description = 'time in unix epoch time (seconds after 1970-01-01 00:00:00 UTC)'
x.description = 'x-coordinate RDNAP for S02_RBR_02 [m]'
y.description = 'y-coordinate RDNAP for S02_RBR_02 [m]'
z.description = 'z-coordinate RDNAP for S02_RBR_02 [m]'
h_bed.description = 'height above the bed of the sensor of S02_RBR_02 [m]'

time.units = 's'
x.units = 'm'
y.units = 'm'
z.units = 'm'
h_bed.units = 'm'

S03_RBR_03 = instrument_heights_jan_2022.createGroup('S03_RBR_03')
S03_RBR_03.description = 'RDNAP coordinates and heights above the bed for S03_RBR_03'

time = S03_RBR_03.createVariable('time', 'i8', 'time')
x = S03_RBR_03.createVariable('x', 'g', 'loc')  
y = S03_RBR_03.createVariable('y', 'g', 'loc')
z = S03_RBR_03.createVariable('z', 'g', 'loc')
h_bed = S03_RBR_03.createVariable('h_bed', 'g', 'time')  

x[:] = '%.4f' % round(S03_RBR_03_gps[0],4)
y[:] = '%.4f' % round(S03_RBR_03_gps[1],4)
z[:] = '%.4f' % round(S03_RBR_03_gps[2],4)

time[:] = time_surveys
h_bed[:] = np.array([-17.5,-20,-21])/100 

time.description = 'time in unix epoch time (seconds after 1970-01-01 00:00:00 UTC)'
x.description = 'x-coordinate RDNAP for S03_RBR_03 [m]'
y.description = 'y-coordinate RDNAP for S03_RBR_03 [m]'
z.description = 'z-coordinate RDNAP for S03_RBR_03 [m]'
h_bed.description = 'height above the bed of the sensor of S03_RBR_03 [m]'

time.units = 's'
x.units = 'm'
y.units = 'm'
z.units = 'm'
h_bed.units = 'm'

S04_RBR_04 = instrument_heights_jan_2022.createGroup('S04_RBR_04')
S04_RBR_04.description = 'RDNAP coordinates and heights above the bed for S04_RBR_04'

time = S04_RBR_04.createVariable('time', 'i8', 'time')
x = S04_RBR_04.createVariable('x', 'g', 'loc')  
y = S04_RBR_04.createVariable('y', 'g', 'loc')
z = S04_RBR_04.createVariable('z', 'g', 'loc')
h_bed = S04_RBR_04.createVariable('h_bed', 'g', 'time')  

x[:] = '%.4f' % round(S04_RBR_04_gps[0],4)
y[:] = '%.4f' % round(S04_RBR_04_gps[1],4)
z[:] = '%.4f' % round(S04_RBR_04_gps[2],4)

time[:] = time_surveys
h_bed[:] = np.array([34,14,27])/100 

time.description = 'time in unix epoch time (seconds after 1970-01-01 00:00:00 UTC)'
x.description = 'x-coordinate RDNAP for S04_RBR_04 [m]'
y.description = 'y-coordinate RDNAP for S04_RBR_04 [m]'
z.description = 'z-coordinate RDNAP for S04_RBR_04 [m]'
h_bed.description = 'height above the bed of the sensor of S04_RBR_04 [m]'

time.units = 's'
x.units = 'm'
y.units = 'm'
z.units = 'm'
h_bed.units = 'm'

S05_RBR_05 = instrument_heights_jan_2022.createGroup('S05_RBR_05')
S05_RBR_05.description = 'RDNAP coordinates and heights above the bed for S05_RBR_05'

time = S05_RBR_05.createVariable('time', 'i8', 'time')
x = S05_RBR_05.createVariable('x', 'g', 'loc')  
y = S05_RBR_05.createVariable('y', 'g', 'loc')
z = S05_RBR_05.createVariable('z', 'g', 'loc')
h_bed = S05_RBR_05.createVariable('h_bed', 'g', 'time')  

x[:] = '%.4f' % round(S05_RBR_05_gps[0],4)
y[:] = '%.4f' % round(S05_RBR_05_gps[1],4)
z[:] = '%.4f' % round(S05_RBR_05_gps[2],4)

time[:] = time_surveys
h_bed[:] = np.array([18,25,30])/100 

time.description = 'time in unix epoch time (seconds after 1970-01-01 00:00:00 UTC)'
x.description = 'x-coordinate RDNAP for S05_RBR_05 [m]'
y.description = 'y-coordinate RDNAP for S05_RBR_05 [m]'
z.description = 'z-coordinate RDNAP for S05_RBR_05 [m]'
h_bed.description = 'height above the bed of the sensor of S05_RBR_05 [m]'

time.units = 's'
x.units = 'm'
y.units = 'm'
z.units = 'm'
h_bed.units = 'm'

S06_RBR_06 = instrument_heights_jan_2022.createGroup('S06_RBR_06')
S06_RBR_06.description = 'RDNAP coordinates and heights above the bed for S06_RBR_06'

time = S06_RBR_06.createVariable('time', 'i8', 'time')
x = S06_RBR_06.createVariable('x', 'g', 'loc')  
y = S06_RBR_06.createVariable('y', 'g', 'loc')
z = S06_RBR_06.createVariable('z', 'g', 'loc')
h_bed = S06_RBR_06.createVariable('h_bed', 'g', 'time')  

x[:] = '%.4f' % round(S06_RBR_06_gps[0],4)
y[:] = '%.4f' % round(S06_RBR_06_gps[1],4)
z[:] = '%.4f' % round(S06_RBR_06_gps[2],4)

time[:] = time_surveys
h_bed[:] = np.array([-18,1,0])/100 

time.description = 'time in unix epoch time (seconds after 1970-01-01 00:00:00 UTC)'
x.description = 'x-coordinate RDNAP for S06_RBR_06 [m]'
y.description = 'y-coordinate RDNAP for S06_RBR_06 [m]'
z.description = 'z-coordinate RDNAP for S06_RBR_06 [m]'
h_bed.description = 'height above the bed of the sensor of S06_RBR_06 [m]'

time.units = 's'
x.units = 'm'
y.units = 'm'
z.units = 'm'
h_bed.units = 'm'

S01_JFE_01 = instrument_heights_jan_2022.createGroup('S01_JFE_01')
S01_JFE_01.description = 'RDNAP coordinates and heights above the bed for S01_JFE_01'

time = S01_JFE_01.createVariable('time', 'i8', 'time')
x = S01_JFE_01.createVariable('x', 'g', 'loc')  
y = S01_JFE_01.createVariable('y', 'g', 'loc')
z = S01_JFE_01.createVariable('z', 'g', 'loc')
h_bed = S01_JFE_01.createVariable('h_bed', 'g', 'time')  

x[:] = '%.4f' % round(S01_JFE_01_gps[0],4)
y[:] = '%.4f' % round(S01_JFE_01_gps[1],4)
z[:] = '%.4f' % round(S01_JFE_01_gps[2],4)

time[:] = time_surveys
h_bed[:] = np.array([25,20,25])/100 

time.description = 'time in unix epoch time (seconds after 1970-01-01 00:00:00 UTC)'
x.description = 'x-coordinate RDNAP for S01_JFE_01 [m]'
y.description = 'y-coordinate RDNAP for S01_JFE_01 [m]'
z.description = 'z-coordinate RDNAP for S01_JFE_01 [m]'
h_bed.description = 'height above the bed of the sensor of S01_JFE_01 [m]'

time.units = 's'
x.units = 'm'
y.units = 'm'
z.units = 'm'
h_bed.units = 'm'

S02_JFE_02 = instrument_heights_jan_2022.createGroup('S02_JFE_02')
S02_JFE_02.description = 'RDNAP coordinates and heights above the bed for S02_JFE_02'

time = S02_JFE_02.createVariable('time', 'i8', 'time')
x = S02_JFE_02.createVariable('x', 'g', 'loc')  
y = S02_JFE_02.createVariable('y', 'g', 'loc')
z = S02_JFE_02.createVariable('z', 'g', 'loc')
h_bed = S02_JFE_02.createVariable('h_bed', 'g', 'time')  

x[:] = '%.4f' % round(S02_JFE_02_gps[0],4)
y[:] = '%.4f' % round(S02_JFE_02_gps[1],4)
z[:] = '%.4f' % round(S02_JFE_02_gps[2],4)

time[:] = time_surveys
h_bed[:] = np.array([17,10,7])/100 

time.description = 'time in unix epoch time (seconds after 1970-01-01 00:00:00 UTC)'
x.description = 'x-coordinate RDNAP for S02_JFE_02 [m]'
y.description = 'y-coordinate RDNAP for S02_JFE_02 [m]'
z.description = 'z-coordinate RDNAP for S02_JFE_02 [m]'
h_bed.description = 'height above the bed of the sensor of S02_JFE_02 [m]'

time.units = 's'
x.units = 'm'
y.units = 'm'
z.units = 'm'
h_bed.units = 'm'

S04_JFE_03 = instrument_heights_jan_2022.createGroup('S04_JFE_03')
S04_JFE_03.description = 'RDNAP coordinates and heights above the bed for S04_JFE_03'

time = S04_JFE_03.createVariable('time', 'i8', 'time')
x = S04_JFE_03.createVariable('x', 'g', 'loc')  
y = S04_JFE_03.createVariable('y', 'g', 'loc')
z = S04_JFE_03.createVariable('z', 'g', 'loc')
h_bed = S04_JFE_03.createVariable('h_bed', 'g', 'time')  

x[:] = '%.4f' % round(S04_JFE_03_gps[0],4)
y[:] = '%.4f' % round(S04_JFE_03_gps[1],4)
z[:] = '%.4f' % round(S04_JFE_03_gps[2],4)

time[:] = time_surveys
h_bed[:] = np.array([23,28,38])/100 

time.description = 'time in unix epoch time (seconds after 1970-01-01 00:00:00 UTC)'
x.description = 'x-coordinate RDNAP for S04_JFE_03 [m]'
y.description = 'y-coordinate RDNAP for S04_JFE_03 [m]'
z.description = 'z-coordinate RDNAP for S04_JFE_03 [m]'
h_bed.description = 'height above the bed of the sensor of S04_JFE_03 [m]'

time.units = 's'
x.units = 'm'
y.units = 'm'
z.units = 'm'
h_bed.units = 'm'

S05_JFE_04 = instrument_heights_jan_2022.createGroup('S05_JFE_04')
S05_JFE_04.description = 'RDNAP coordinates and heights above the bed for S05_JFE_04'

time = S05_JFE_04.createVariable('time', 'i8', 'time')
x = S05_JFE_04.createVariable('x', 'g', 'loc')  
y = S05_JFE_04.createVariable('y', 'g', 'loc')
z = S05_JFE_04.createVariable('z', 'g', 'loc')
h_bed = S05_JFE_04.createVariable('h_bed', 'g', 'time')  

x[:] = '%.4f' % round(S05_JFE_04_gps[0],4)
y[:] = '%.4f' % round(S05_JFE_04_gps[1],4)
z[:] = '%.4f' % round(S05_JFE_04_gps[2],4)

time[:] = time_surveys
h_bed[:] = np.array([29,35,38])/100 

time.description = 'time in unix epoch time (seconds after 1970-01-01 00:00:00 UTC)'
x.description = 'x-coordinate RDNAP for S05_JFE_04 [m]'
y.description = 'y-coordinate RDNAP for S05_JFE_04 [m]'
z.description = 'z-coordinate RDNAP for S05_JFE_04 [m]'
h_bed.description = 'height above the bed of the sensor of S05_JFE_04 [m]'

time.units = 's'
x.units = 'm'
y.units = 'm'
z.units = 'm'
h_bed.units = 'm'

#%%

instrument_heights_jan_2022.close()

#%%
instrument_heights_jan_2022 = nc4.Dataset(r'20220105_20220107_instrument_heights.nc','r')

#%%

print(pd.to_datetime(instrument_heights_jan_2022['S01_RBR_01']['time'][:], unit='s' ))
print(instrument_heights_jan_2022['S01_RBR_01']['h_bed'][1])
print(instrument_heights_jan_2022['S02_RBR_02']['h_bed'][2])
print(instrument_heights_jan_2022['S03_RBR_03']['h_bed'][:])
print(instrument_heights_jan_2022['S04_RBR_04']['h_bed'][:])
print(instrument_heights_jan_2022['S05_RBR_05']['h_bed'][:])
print(instrument_heights_jan_2022['S06_RBR_06']['h_bed'][:])

print(instrument_heights_jan_2022['S01_RBR_01']['z'][:])
print(instrument_heights_jan_2022['S02_RBR_02']['z'][:])
print(instrument_heights_jan_2022['S03_RBR_03']['z'][:])
print(instrument_heights_jan_2022['S04_RBR_04']['z'][:])
print(instrument_heights_jan_2022['S05_RBR_05']['z'][:])
print(instrument_heights_jan_2022['S06_RBR_06']['z'][:])
print(instrument_heights_jan_2022['S01_JFE_01']['z'][:])


#%%

instrument_heights_jan_2022.close()


#%%


